@@ -1,4 +1,4 @@
-# $Id: Constants.pm 380 2005-03-10 02:59:51Z claco $
+# $Id: Constants.pm 768 2005-09-05 02:25:24Z claco $
 package Template::Plugin::Handel::Constants;
 use strict;
 use warnings;
@@ -29,23 +29,44 @@ __END__
 
 =head1 NAME
 
-Template::Plugin::Handel::Constants - Template Toolkit plugin for Handel constants
+Template::Plugin::Handel::Constants - Template Toolkit plugin for Handel
+constants
 
 =head1 SYNOPSIS
 
     [% USE hdl = Handel.Constants %]
     [% hdl.CART_TYPE_SAVED %]
 
+    or
+
+    [% USE Handel.Constants %]
+    [% Handel.Constants.CART_TYPE_SAVED %]
+
 =head1 DESCRIPTION
 
-C<Template::Plugin::Handel::Constants> is a TT2 (Template Toolkit 2) plugin for
-Handel::Constants.
+C<Template::Plugin::Handel::Constants> is a TT2 (Template Toolkit 2) plugin to
+access C<Handel::Constants> iside of TT2 pages.
 
-It contains all of the exportable constants declared in @Handel::Constants::EXPORT_OK.
+It contains all of the exportable constants declared in
+C<@Handel::Constants::EXPORT_OK>.
+
+=head1 CONSTRUCTOR
+
+=head2 new
+
+This returns a new Handel.Constants object. This is used internally when
+loading TT2 plugins and should not be used directly.
+
+=head1 METHODS
+
+=head2 load
+
+This method is called when TT2 loaded the plugin for the first time.
+This is used internally by TT2 and should not be used directly.
 
 =head1 SEE ALSO
 
-L<Handel::Constants>
+L<Handel::Constants>, L<Template::Plugin>
 
 =head1 AUTHOR
 
